
@GENERALFAMILY_LIST = ('Roman', 'SansSerif', 'Typewriter', 'Symbol',
			 'Gothic', 'Mincho');
@WEIGHT_LIST = ('Medium', 'Bold', 'Semibold', 'Light', 'Semilight');
@SHAPE_LIST = ('Serif', 'NoSerif', 'Upright', 'Oblique', 'Italic',
	       'Condensed', 'Expanded');

$ROOTDIR = "$DEFOMA_TEST_DIR/var/lib/defoma";

$DIALOGTITLE = '';
$DWIDTH = 70;

sub linecount {
    my $text = shift;
    my $lines = 0;
    my $cnt = 0;
    my @words = split(/ /, $text);
    my($i, $len, $space);

    foreach $i (@words) {
	if ($i eq '\n') {
	    $cnt = 0;
	    next;
	}
	
	$len = length($i);
	while (1) {
	    $lines++ if ($cnt == 0);
	    $space = ($cnt > 0);
	    
	    if ($cnt + $space + $len > $DWIDTH) {
		if ($cnt == 0) {
		    $len -= $DWIDTH;
		} else {
		    $cnt = 0;
		}
	    } else {
		$cnt += $len;
		last;
	    }
	}
    }

    return $lines;
}

sub callsystem {
    my $redirect = shift;
    my @args = @_;
    my $i;
    my $arg = '';

    for ($i = 0; $i < @_; $i++) {
	$arg .= " '$args[$i]'";
    }

    return system "$arg $redirect";
}

sub callsystem_output {
    my $redirect = shift;
    my $tempfile = `tempfile`;
    chomp($tempfile);
    push(@EXITREMOVE, $tempfile);
    my $result;

    $redirect = "1> $tempfile $redirect";

    $result = callsystem($redirect, @_);
    $output = `cat $tempfile`;

    unlink $tempfile;
    pop(@EXITREMOVE);

    return $result;
}

# code from pppconfig.
#
#   Copyright (C) 1999 John G. Hasler (john@dhh.gt.org)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

sub dialogbox(@) {
  my $type=shift(@_);
  my $optionstr = (@_ > 0) ? shift(@_) : '';
  my @vars=@_;
  my $text=shift( @vars );
  my $height = shift( @vars );
  my $title = $DIALOGTITLE;

  my @options = split(/ +/, $optionstr) if ($optionstr ne '');

  $result = callsystem_output('2>&1', '/usr/bin/dialog', '--clear', '--title',
			      $title, @options, $type, $text, $height, 80,
			      @vars);
  my $item=$output;
  chomp $item; # For gdialog, which returns strings with newlines.
  $result = ($result >> 8);
  exitfunc(255) if ($result == 255);
  die("Internal error ") unless($result == 0 || $result == 1);
  return $item;
}

sub msgbox(@) {
    my $text = shift;
    my $lines = linecount($text);
    $lines += 7;

    dialogbox("\-\-msgbox", '', $text, $lines);

    return $result;
}

sub infobox(@) {
    my $text = shift;
    my $lines = linecount($text);
    $lines += 7;

    dialogbox("\-\-infobox", '', $text, $lines);

    return $result;
}

sub yesnobox(@) {
    my $text = shift;
    my $lines = linecount($text);
    $lines += 7;
    
    dialogbox( "\-\-yesno", '', $text, $lines);

    return $result;
}

sub inputbox(@) {
    my $text = shift;
    my $default = shift;
    my $pattern = shift;
    my $allowempty = shift;
    my $lines = linecount($text);
    my @args;
    my $ret;
    $lines += 7;

    while(1) {
	@args = ();
	push(@args, $text);
	push(@args, $lines);
	push(@args, $default) if ($default ne '');
	
	$ret = dialogbox( "\-\-inputbox", '', @args);

	return '' if ($result != 0);
	return '' if ($ret eq '' && $allowempty != 0);
	return $ret if ($ret =~ /^$pattern+$/);

	if ($ret eq '') {
	    $text = "Empty is not allowed.";
	} else {
	    $default = $ret;
	    $ret =~ s/$pattern//g;
	    $default =~ s/[^$ret]/_/g;
	    $text = "Illegal characters: \"$ret\".";
	    if ($ret =~ / /) {
		$text .= "\n you can use underscore in place of space.";
	    }
	}
	$lines = 8;
    }
}

sub menu(@) {
    my $text = shift( @_ );
    my $menu_height = shift( @_ );
    my $options = shift;
    my $lines = linecount($text);
    $lines += 6 + $menu_height;
    
    return dialogbox( '--menu', $options, $text, $lines, $menu_height, @_ );
}

sub menu_single(@) {
    my $text = shift( @_ );
    my $menu_height = shift( @_ );
    my $options = shift;
    my $lines = linecount($text);
    $lines += 6 + $menu_height;
    
    my @args = @_;
    my @pass = ();
    my $i;

    for ($i = 0; $i < @args; $i++) {
	if ($args[$i] ne '') {
	    $pass[$i * 2] = $args[$i];
	    $pass[$i * 2 + 1] = ' ';
	}
    }
    
    return dialogbox( '--menu', $options, $text, $lines, $menu_height,
		      @pass );
}

sub checklist_single(@) {
    my $text = shift( @_ );
    my $menu_height = shift( @_ );
    my $lines = linecount($text);
    $lines += 6 + $menu_height;
    
    my @args = @_;
    my @pass = ();
    my $i;
    my $j;

    for ($i = $j = 0; $i < @args; ) {
	if ($args[$i] ne '') {
	    $pass[$j++] = $args[$i++];
	    $pass[$j++] = ' ';
	    $pass[$j++] = $args[$i++];
	}
    }
    
    return dialogbox('--checklist', '', $text, $lines, $menu_height, @pass);
}

sub checklist_single_onargs(@) {
    my $text = shift( @_ );
    my $menu_height = shift( @_ );
    my $onargs = shift;
    my $lines = linecount($text);
    $lines += 6 + $menu_height;
    
    my @args = @_;
    my @pass = ();
    my $i;
    my $j;
    my $pattern = join('|', split(/ +/, $onargs));

    for ($i = $j = 0; $i < @args; $i++) {
	if ($args[$i] ne '') {
	    $pass[$j++] = $args[$i];
	    $pass[$j++] = ' ';
	    $pass[$j++] = ($args[$i] =~ /^($pattern)$/) ? 'on' : 'off';
	}
    }
    
    return dialogbox('--checklist', '--separate-output', $text, $lines,
		     $menu_height, @pass );
}

sub fileselector {
    my $text = shift;
    my $origdir = `pwd`;
    chomp($origdir);
    my $file;
    my $retfile = '';

    my $dtitle = $DIALOGTITLE;
    $DIALOGTITLE = 'File Selector';

    while (1) {
	my $dir=`pwd`;
	chomp($dir);

	my @dirs = ();
	my @files = ();
	my @list;

	opendir(DIR, '.');
	@list = readdir(DIR);
	closedir(DIR);

	foreach $file (@list) {
	    next if ($file eq '.');
	    
	    if (-d $file) {
		push(@dirs, "$file/");
	    } else {
		push(@files, $file);
	    }
	}
	
	my $ddir = $dir;
	my $len = length($ddir);
	if ($len > 60) {
	    $len -= 60;
	    $ddir =~ s/^.{$len}//;
	}

	my $desc = "$text\\n\\nDir: $ddir";
	$file = menu_single($desc, 10, '', @dirs, @files);
	$file =~ s@/$@@;

	last if ($result != 0);

	if (-d $file) {
	    chdir $file;
	} else {
	    $retfile = "$dir/$file";
	    last;
	}
    }

    $DIALOGTITLE = $dtitle;
    chdir $origdir;

    return $retfile;
}
	
my @PARSEHINTS_KEY;
my @PARSEHINTS_LIST;
my @PARSEHINTS_OPTION;

sub parse_all_hints_conf {
    my $key = shift;
    my $listptr = shift;
    my $option = (@_ > 0) ? shift(@_) : '';
    my $i = @PARSEHINTS_KEY;

    $PARSEHINTS_KEY[$i] = $key;
    $PARSEHINTS_LIST[$i] = $listptr;
    $PARSEHINTS_OPTION[$i] = $option;
}

sub parse_all_hints {
    my $allhints;
    my $hints;

    $allhints = `cut -d ' ' -f 2- $ROOTDIR/*.font-cache 2> /dev/null`;

    while ($allhints ne '') {
	$allhints =~ s/^.*\n//m;
	$hints = $&;
	chomp($hints);
	
	for (my $i = 0; $i < @PARSEHINTS_KEY; $i++) {
	    my $key = $PARSEHINTS_KEY[$i];
	    my $listptr = $PARSEHINTS_LIST[$i];
	    my $pattern = join('|', @{$listptr});
	    	
	    if ($hints =~ /--$key +[^ ]+/) {
		my $values = $hints;
		$values =~ s/.*--$key +(.*)/$1/;
		$values =~ s/ +--.*//;

		@valuelist = split(/ +/, $values);
		
		for (my $j = 0; $j < @valuelist; $j++) {
		    if ($valuelist[$j] !~ /$pattern/) {
			push(@{$listptr}, $valuelist[$j]);
			if ($PARSEHINTS_OPTION[$i] ne '') {
			    push(@{$listptr}, $PARSEHINTS_OPTION[$i]);
			}
		    }
		}
	    }
	}
    }
}

sub parse_all_hints_done {
    @PARSEHINTS_KEY = ();
    @PARSEHINTS_LIST = ();
    @PARSEHINTS_OPTION = ();
}

$INPUTCOMMON_MENU = 1;

sub input_menu {
    my $input_text = shift;
    my $default = shift;
    my $input_pattern = shift;
    my $input_allowempty = shift;
    my $input_menu_item = '';
    my $menu_text = '';
    my @menu_list = ();
    if (@_ >= 3) {
	$input_menu_item = shift;
	$menu_text = shift;
	@menu_list = @_;
    }

    while (1) {
	if (@menu_list > 0) {
	    chomp($menu_text);
	    my $lines = 1;
	    while ($menu_text =~ /\n/m) {
		$menu_text =~ s/\n/\\n/m;
		$lines++;
	    }
	    
	    my $menu_items = @menu_list;
	    my $mlines = 15 - $lines;
	    my $items = ($menu_items > $mlines) ? $mlines : $menu_items;
	    my $default_item = '';
	    $default_item = "--default-item $default" if ($default ne '');
	    my $ret;

	    if ($INPUTCOMMON_MENU == 1) {
		$ret = menu_single($menu_text, $items, $default_item,
				      @menu_list);
	    } else {
		$ret = menu($menu_text, $items, $default_item, @menu_list);
	    }

	    return '' if ($result != 0);
	    return $ret if ($ret ne $input_menu_item);
	}
	
	chomp($input_text);
	$input_text =~ s/\n/\\n/gm;

	$ret = inputbox($input_text, $default, $input_pattern,
			$input_allowempty);
	return $ret if ($result == 0 || $menu_text eq '');
    }
}

sub input_checklist {
    my $input_text = shift;
    my $default = shift;
    my $input_pattern = shift;
    my $input_allowempty = shift;
    my $clist_text = '';
    my @clist_list = ();
    if (@_ > 0) {
	$clist_text = shift;
	@clist_list = @_;
    }

    while (1) {
	chomp($clist_text);
	my $lines = 1;
	while ($clist_text =~ /\n/m) {
	    $clist_text =~ s/\n/\\n/m;
	    $lines++;
	}
	
	my $clist_items = @clist_list;
	my $clines = 15 - $lines;
	my $items = ($clist_items > $clines) ? $clines : $clist_items;
	my $ret;
	
	$ret = checklist_single_onargs($clist_text, $items, $default,
				       @clist_list);
	return '' if ($result != 0);
	chomp($ret);
	$ret =~ s/\n/ /g;
    	
	$ret = inputbox($input_text, $ret, $input_pattern, $input_allowempty);
	return $ret if ($result == 0);
    }
}


sub input_menu2 {
    $INPUTCOMMON_MENU = 2;
    my $ret = input_menu(@_);
    $INPUTCOMMON_MENU = 1;
    return $ret;
}

sub input_fontname {
    my $default = shift;
    my $text = <<EOF
Input the FontName of the font.
* FontName should be and must be a font-specific identifier. For example,
* a font of FooBar family, Bold weight and Italic shape should have
* FooBar-BoldItalic as the FontName.
EOF
    ;

    return input_menu($text, $default, '[^ \t]', 0);
}

sub input_family {
    my $font = shift;
    my $default = shift;
    my $text = <<EOF
Input the Family of $font.
* Family of the font is similar to a family name of a person. A font
* often has some decorated derivative fonts, but all of the derivative
* fonts and its original font share a common name. Family is exactly
* the shared common name. For example, Times-Roman has three decorated
* versions, Times-Italic, Times-Bold and Times-BoldItalic, and Family
* of them is Times.
EOF
    ;

    return input_menu($text, $default, '[^ \t]', 0);
}

%F2G = ();

sub input_generalfamily {
    my $font = shift;
    my $family = shift;
    my $text = <<EOF
Choose the GeneralFamily of $font.
* GeneralFamily represents the rough group which the font belongs to.
* This hint is useful for substitution of fonts, because fonts which
* belong to the same GeneralFamily are supposed to have more similar
* font faces than those which do not.
Following is a list of standard General Families (Roman, SansSerif,
Typewriter, Symbol, Gothic, and Mincho) and already registered General
Families. Please choose GeneralFamily from the list, or None if you
want to input a new GeneralFamily manually.
EOF
    ;

    my $default = exists($F2G{$family}) ? $F2G{$family} : '';

    my $ret = input_menu('Input the GeneralFamily of the font manually.',
			 $default, '[^ \t]', 0, '<None>', $text,
			 @GENERALFAMILY_LIST, '<None>');
    if ($result == 0) {
	$F2G{$family} = $ret;
    }

    return $ret;
}

sub input_weight {
    my $font = shift;
    my $default = shift;
    my $menutext = <<EOF
Choose the Weight of $font.
* Weight represends the heaviness of the appearance, or the thickness
* of lines of gryphs, of the font.
Following is a list of standard Weights (Medium, Bold, Semibold, Light,
and Semilight) and already registered Weights. Please choose Weight from
the list, or None if you want to input a new Weight manually.
EOF
    ;

    return input_menu('Input the Weight of the font manually.', $default,
		      '[^ \t]', 0, '<None>', $menutext, @WEIGHT_LIST,
		      '<None>');
}

sub input_width {
    my $font = shift;
    my $default = shift;
    my $menutext = <<EOF
Choose the Width of $font.
* Width specifies whether the width of gryphs of the font varies, or is
* fixed. Typewriter fonts are maybe famous fixed width fonts. Most Latin
* fonts are variable width ones. Kanji fonts are also regarded as fixed
* width.
EOF
    ;

    return input_menu('', $default, '', 0, '', $menutext, 'Variable', 'Fixed');
}

sub input_shape {
    my $font = shift;
    my $default = shift;
    my @dlist = split(/ +/, $default);
    my $slant = '';
    my $serif = '';
    my $width = '';
    my $ret;

    for (my $i = 0; $i < @dlist; $i++) {
	$slant = $dlist[$i] if ($dlist[$i] =~ /^(Upright|Italic|Oblique)$/);
	$width = $dlist[$i] if ($dlist[$i] =~ /^(Condensed|Expanded)$/);
	$serif = $dlist[$i] if ($dlist[$i] =~ /^(Serif|NoSerif)$/);
    }

    $width = 'Normal' if ($serif eq '');
    $slant = 'Upright' if ($slant eq '');

    my $text = <<EOF
Choose the Shapes of $font.
* Shape represents additional information about the appearance of gryphs
* of the font. This Hint category consists of several types of font faces,
* including Serif, Slant, and the extent of Width. The last one, Width
* hint here is absolutely different from Fixed/Variable Width hint, which
* is supposed to be already chosen.
Following is a list of candidates of hints about Shape of the font. Mark
the hints applicable to the font, by Space key.
EOF
    ;
    $text =~ s/\n/\\n/gm;

    $ret = checklist_single_onargs($text, 9, "$width $slant $serif",
				   @SHAPE_LIST);

    my @hlist = split(/\n/, $ret);
    $slant = '';
    $width = '';
    $serif = '';
    
    for ($i = 0; $i < @hlist; $i++) {
	if ($hlist[$i] =~ /^(Upright|Oblique|Italic)$/) {
	    if ($slant eq '') {
		$slant = $hlist[$i];
	    } elsif ($slant =~ /^(Oblique|Italic)$/ &&
		     $hlist[$i] =~ /^(Oblique|Italic)$/) {
		$slant = 'Italic';
	    } else {
		$text = "$slant and $hlist[$i] confclicts. ";
		$text .= "Which is correct?";
		$slant = menu_single($text, 2, '', $slant, $hlist[$i]);
	    }
	    $hlist[$i] = '';
	}
	if ($hlist[$i] =~ /^(Expanded|Condensed)$/) {
	    if ($width eq '') {
		$width = $hlist[$i];
	    } else {
		$text = "$width and $hlist[$i] confclicts.";
		$text .= "Which is correct?";
		$width = menu_single($text, 2, '', $width, $hlist[$i]);
	    }
	    $hlist[$i] = '';
	}
	if ($hlist[$i] =~ /^(Serif|NoSerif)$/) {
	    if ($serif eq '') {
		$serif = $hlist[$i];
	    } else {
		$text = "$serif and $hlist[$i] confclicts.";
		$text .= "Which is correct?";
		$serif = menu_single($text, '', $serif, $hlist[$i]);
	    }
	    $hlist[$i] = '';
	}
    }

    $default = join(' ', @hlist, $serif, $slant, $width);
    $default =~ s/ +/ /g;
    
    return input_menu('Add the Shapes of the font.', $default, '.', 1);
}

sub input_alias {
    my $font = shift;
    my $default = shift;
    my $text = <<EOF
Input the Alias(es) of $font, if exists. 
* Alias represents other FontName(s) of a font. Specifying them will
* make the font accessible by the alias(es).
You can specify more than one aliases by separating them by space.
EOF
    ;

    return input_menu($text, $default, '[^ \t]', 1);
}

sub input_priority {
    my $font = shift;
    my $default = shift;
    my $text = <<EOF
Input the Priority of $font between 0 and 99.
* Priority is used when more than one fonts provide the same identifier
* in ID cache. The font which has the largest Priority of them will 
* actually get installed.
EOF
    ;

    return input_menu($text, $default, '[0-9]', 0);
}

sub hint_beginlib {
    parse_all_hints();
    parse_all_hints_done();

    $DIALOGTITLE = shift;
    $DWIDTH = shift;
}

BEGIN {
    parse_all_hints_conf('GeneralFamily', \@GENERALFAMILY_LIST);
    parse_all_hints_conf('Weight', \@WEIGHT_LIST);
    parse_all_hints_conf('Shape', \@SHAPE_LIST);
}

1;
