package Debian::Defoma::FontCache;
use strict;
use POSIX;

my $rootdir = '';

sub initialize {
    $rootdir = shift;
}

sub new {
    my $class = shift;
    my $c;

    my $o = {
	category => shift,
	cache_list => {},
	fcache_list => {}
    };

    bless $o;
    return $o;
}

sub read {
    my $o = shift;
    my $c = $o->{category};

    my $file = "$rootdir/$c.font-cache";
    if (open(F, $file)) {
	while (<F>) {
	    $_ =~ /^([^ ]+)[ ]+(.*)$/;
	    $o->{cache_list}->{$1} = $2;
	}
	close F;
    }
    
    $file = "$rootdir/$c.failed-font-cache";
    if (open(F, $file)) {
	while (<F>) {
	    chomp($_);
	    $_ =~ /^([^ ]+) ([^ ]+) ([^ ]+)$/;
	    unless (exists($o->{fcache_list}->{$1})) {
		$o->{fcache_list}->{$1} = {};
	    }
	    $o->{fcache_list}->{$1}->{$2} = $3;
	}
	close F;
    }

    return 0;
}

sub write {
    my $o = shift;
    my $c = $o->{category};
    my ($a, $max, $f);
    my @fonts;

    my $file = "$rootdir/$c.font-cache";

    if (open(F, '>' . $file)) {
	@fonts = keys(%{$o->{cache_list}});
	foreach $f (@fonts) {
	    print F $f, ' ', $o->{cache_list}->{$f}, "\n";
	}
	close F;
    }
    unlink($file) unless(-s $file);

    $file = "$rootdir/$c.failed-font-cache";

    if (open(F, '>' . $file)) {
	@fonts = keys(%{$o->{fcache_list}});
	foreach $f (@fonts) {
	    my @apps = keys(%{$o->{fcache_list}->{$f}});
	    foreach $a (@apps) {
		print F $f, ' ', $a, ' ', $o->{fcache_list}->{$f}->{$a}, "\n";
	    }
	}
	close F;
    }
    unlink($file) unless(-s $file);

    return 0;
}

sub add_font {
    my $o = shift;
    my $font = shift;
    my @hints = @_;

    $o->{cache_list}->{$font} = join(' ', @hints);

    return 0;
}

sub add_failed {
    my $o = shift;
    my $f = shift;
    my $a = shift;
    my $e = shift;

    $o->{fcache_list}->{$f} = {} unless (exists($o->{fcache_list}->{$f}));
    $o->{fcache_list}->{$f}->{$a} = $e;
    
    return 0;
}

sub remove_font {
    my $o = shift;
    my $f = shift;

    delete($o->{cache_list}->{$f});

    return 0;
}

sub remove_failed {
    my $o = shift;
    my $f = shift;
    my $a = shift;

    if (exists($o->{fcache_list}->{$f}->{$a})) {
	delete($o->{fcache_list}->{$f}->{$a});
	return 1;
    }
    return 0;
}

1;
