package Debian::Defoma::Id;

use strict;
use POSIX;
use Exporter;

use vars qw(@EXPORT @EXPORT_FAIL @ISA $DEFAULT_PACKAGE $DEFAULT_CATEGORY
	    $Purge);

use Debian::Defoma::Common;
use Debian::Defoma::IdCache;
import Debian::Defoma::Common qw(&register_id_object &get_id_object);
import Debian::Defoma::Common qw($DEFAULT_PACKAGE $DEFAULT_CATEGORY);


@ISA = qw(Exporter);

@EXPORT = qw(&defoma_id_register &defoma_id_unregister
	     &defoma_id_open_cache &defoma_id_close_cache
	     &defoma_id_get_font &defoma_id_update);

my %TYPE = ('r' => 'real', 'a' => 'alias', 'S' => 'subst');
my %RTYPE = ('real' => 'Sr', 'alias' => 'Sa', 'subst' => 'SS');

$Purge = 0;

sub emes {
    my $o = shift;
    my $pkg = $o->{pkg} || $DEFAULT_PACKAGE || 'Unknown';
    
    printw("Id: $pkg: ", @_);
}

sub emesd {
    my $o = shift;
    my $pkg = $o->{pkg} || $DEFAULT_PACKAGE || 'Unknown';
    
    printd("Id: $pkg: ", @_);
}

my $Obj;

sub sorter {
    my $a1 = $Obj->{2}->[$a];
    $a1 =~ s/(..)(.*)/$1/;
    my $a2 = $2;
    my $a3 = $Obj->{3}->[$a];
  
    my $b1 = $Obj->{2}->[$b];
    $b1 =~ s/(..)(.*)/$1/;
    my $b2 = $2;
    my $b3 = $Obj->{3}->[$b];
    
    return 1 if ($a1 lt $b1);
    return -1 if ($b1 lt $a1);

    if ($a2 ne '' && $b2 ne '') {
	return 1 if ($a2 lt $b2);
	return -1 if ($b2 lt $a2);
    }

    return 1 if ($a3 < $b3);
    return -1 if ($b3 < $a3);
    
    return 0;
}

sub check_if_installed {
    my $obj = shift;
    my $id = shift;
    my $font = shift;
    my @ret = ();

    if ($id ne '.' && $font ne '.') {
	return $obj->grep('installed', f0 => $id, f1 => $font);
    } elsif ($id ne '.' && $font eq '.') {
	return $obj->grep('installed', f0 => $id);
    } elsif ($id eq '.' && $font ne '.') {
	return $obj->grep('installed', f1 => $font);
    }

    return @ret;
}

sub get_top_prior {
    my $obj = shift;
    my $id = shift;
    my @index;
    my %exclude = ();
    my $i;
    my $j;
    my @list;

    # Check marked fonts providing the id first.
    @index = $obj->grep('mark', f0 => $id);
    @index = sort (@index);

    foreach $i (@index) {
	# If a certain font is marked as 'use'...
	if ($obj->{2}->[$i] eq 'Mu') {
	    # Check if the font is actually registered,
	    @list = $obj->grep('font', f1 => $obj->{1}->[$i],
			       f0 => $obj->{0}->[$i]);
	    if (@list) {
		# In case the font is registered:
		$j = $list[0];

		if ($obj->{5}->[$j] ne '.' || $obj->{6}->[$j] ne '.') {
		    # If the font depends on another font, and that font
		    # is not actually installed, ignore the 'use' flag.
		    next unless (check_if_installed($obj, $obj->{5}->[$j],
						    $obj->{6}->[$j]));
		}
		
		return $j;
	    }
	} elsif ($obj->{2}->[$i] eq 'Mx') {
	    # If a certain font is marked as 'exclude'...
	    $exclude{$obj->{1}->[$i]} = 1;
	}
    }

    # Check all fonts providing the id.
    @index = $obj->grep('font', f0 => $id);
    $Obj = $obj;
    @index = sort sorter (@index);

    foreach $i (@index) {
	next if (exists($exclude{$obj->{1}->[$i]}));
	next if (exists($obj->{unregistering}->{$i}));

	my $did = $obj->{5}->[$i];
	my $dfont = $obj->{6}->[$i];

	if ($did ne '.' || $dfont ne '.') {
	    # If the font depends on another font, and that font is not
	    # actually installed, ignore it.
	    next unless (check_if_installed($obj, $did, $dfont));
	}

	return $i;
    }

    return -1;
}

sub gethints {
    my $obj = shift;
    my $i = shift;

    my $type = $obj->{2}->[$i];
    my $id;
    my $font;

    if ($type =~ /^Sr/) {
	return $obj->{7}->[$i];
    } else {
	$id = $obj->{5}->[$i];
	$font = $obj->{6}->[$i];
    }

    my @list = $obj->grep('real', f0 => $id, f1 => $font);
    if (@list > 0) {
	return $obj->{7}->[$list[0]];
    } else {
	return '';
    }
}

sub call_do_remove {
    my $obj = shift;
    my $i = shift;

    my $id = $obj->{0}->[$i];
    
    $obj->{2}->[$i] =~ /^.(.)I$/;
    my $typestr = $TYPE{$1};

    my $depid = ($obj->{5}->[$i] eq '.') ? '' : $obj->{5}->[$i];
    my $depfont = ($obj->{6}->[$i] eq '.') ? '' : $obj->{6}->[$i];

    my @hints = split(/ +/, gethints($obj, $i));

    &Debian::Defoma::Configure::call_1(0, $obj->{pkg},
				       "do-remove-$typestr", $obj->{4}->[$i],
				       $obj->{1}->[$i], $id, $depfont, $depid,
				       @hints) if ($obj->{callback});

    $obj->uninstall($i);

    do_update_depend($obj, $id, $obj->{1}->[$i]);
}

sub call_do_install {
    my $obj = shift;
    my $i = shift;

    my $id = $obj->{0}->[$i];

    $obj->{2}->[$i] =~ /^.(.)$/;
    my $typestr = $TYPE{$1};
    
    my $depid = ($obj->{5}->[$i] eq '.') ? '' : $obj->{5}->[$i];
    my $depfont = ($obj->{6}->[$i] eq '.') ? '' : $obj->{6}->[$i];

    my @hints = split(/ +/, gethints($obj, $i));

    my $font = $obj->{1}->[$i];
    my $ctg = $obj->{4}->[$i];

    $obj->install($i);
    
    my $ret = 0;
    $ret = &Debian::Defoma::Configure::call_1(0, $obj->{pkg},
					      "do-install-$typestr", $ctg,
					      $font, $id, $depfont, $depid,
					      @hints) if ($obj->{callback});
    
    if ($ret) {
	$obj->uninstall($i);
	
	do_set($obj, $id, $font, 'exclude');
	
	my $text = <<EOF
Following package\'s configuration script returned error($ret) during doing do-install-$typestr.
  Package\: $obj->{pkg}
  Category\: $ctg
  Installing Font\: $font
  Installing ID\: $id
Defoma has set this font as \'exclude\' to keep it from being installed.
You can still have it installed by unsetting the \'exclude\' mark after the cause of the error gets removed.
EOF
    ;
	printw($text);

	do_update($obj, $id);
    } else {
	do_update_depend($obj, $id, $font);
    }
}
    
sub do_update {
    my $obj = shift;
    my $id = shift;
    my $i1 = -1;
    my $i2 = -1;
    my $font1 = '';
    my $font2 = '';
    my ($id2, $type2, $pri2, $cate2, $depid2, $depfont2, $hints2);
    my @list;

    @list = $obj->grep('installed', f0 => $id);
    if (@list > 0) {
	$i1 = $list[0];
	$font1 = $obj->{1}->[$i1];
    }

    $i2 = get_top_prior($obj, $id);
    if ($i2 >= 0) {
	$font2 = $obj->{1}->[$i2];
    }

    return if ($font1 eq $font2);

    if ($i1 >= 0) {
	call_do_remove($obj, $i1);
    }

    return if ($obj->{delay});

    if ($i2 >= 0) {
	call_do_install($obj, $i2);
    }
}

sub do_update_depend {
    my $obj = shift;
    my $id = shift;
    my $font = shift;

    my @list = $obj->grep('font', f5 => $id, f6 => $font, or => 1);
    foreach my $i (@list) {
	next if ($obj->{5}->[$i] ne '.' && $obj->{5}->[$i] ne $id);
	next if ($obj->{6}->[$i] ne '.' && $obj->{6}->[$i] ne $font);
	do_update($obj, $obj->{0}->[$i]);
    }
}

sub do_unset {
    my $obj = shift;
    my $id = shift;
    my $font = shift;

    my @list = $obj->grep('mark', f0 => $id, f1 => $font, r2 => '^M[ux]');
    if (@list > 0) {
	$obj->delete(@list);
    }
}

sub do_set {
    my $obj = shift;
    my $id = shift;
    my $font = shift;
    my $type = shift;
    my $typestr;

    if ($type eq 'use') {
	$typestr = 'Mu';
    } elsif ($type eq 'exclude') {
	$typestr = 'Mx';
    }
    
    if ($type eq 'use') {
	my @list = $obj->grep('mark', f0 => $id, f2 => 'Mu');
	if (@list > 0) {
	    $obj->delete($list[0]);
	}
    }

    return $obj->add($id, $font, $typestr, '-', '-', '-', '-', '-');
}

sub defoma_id_open_cache {
    my $suffix = (@_ > 0) ? shift(@_) : '';
    my $pkg = (@_ > 0) ? shift(@_) : $DEFAULT_PACKAGE;
    my $o;

    $o = get_id_object($pkg, $suffix);
    return $o if ($o);

    my $file = $ROOTDIR . '/' . $pkg . '.d/id-cache';
    $file .= '.' . $suffix if ($suffix);

    $o = new Debian::Defoma::IdCache($file, $pkg, $suffix);
    $o->read();

    register_id_object($o, $pkg, $suffix);

    return $o;
}

sub defoma_id_close_cache {
    my $o = shift;
    if ($o) {
	$o->write();
    }
}

sub defoma_id_register {
    my $obj = shift;
    my %args = @_;

    unless (exists($args{type}) && exists($args{font}) && exists($args{id}) &&
	    exists($args{priority})) {
	emes($obj, "register: Required argument is missing.");
	return -1;
    }

    my $comtype = $args{type};
    my $font = $args{font};
    my $id = $args{id};
    my $priority = $args{priority};

    if ($comtype !~ /^(real|alias|subst)$/) {
	emes($obj, "Unknown type '$comtype'.");
	return -1;
    }
    
    my $type = $RTYPE{$comtype};

    arg_check($id, $font, $priority) || return -1;

    return -1 if ($priority =~ /[^0-9]/);
    $priority = 999 if ($priority >= 1000);
    $priority = 0 if ($priority < 0);

    my $category = $DEFAULT_CATEGORY;
    my $depfont = '.';
    my $depid = '.';
    my $hints = '';
    my $i;
    my $dependflag = 0;
    my @l;

    $category = $args{category} if (exists($args{category}));

    if (exists($args{depend})) {
	@l = split(/ /, $args{depend});
	if (@l == 2) {
	    $dependflag = 1;
	    $depfont = $l[0];
	    $depid = $l[1];
	}
    }

    if (exists($args{origin})) {
	$dependflag = 1;
	$depfont = $font;
	$depid = $args{origin};
    }

    if (exists($args{hints})) {
	if ($type eq 'Sr') {
	    $hints = $args{hints};
	} else {
	    emesd($obj, "register: Only 'real' accepts '--hints'.");
	}
    }

    if ($type ne 'Sr' && $dependflag == 0) {
	emes($obj, "'$comtype' requires --origin be specified.");
	return -1;
    }

    if ($obj->grep('font', f0 => $id, f1 => $font)) {
	emesd($obj, "$id: already registered by $font.");
	return -1;
    }

    $obj->add($id, $font, $type, $priority, $category, $depid, $depfont,
	      $hints);

    do_update($obj, $id);

    return 0;
}

sub defoma_id_unregister {
    my $obj = shift;
    my %args = @_;

    unless (exists($args{type}) && exists($args{font})) {
	emes($obj, "register: Required argument is missing.");
	return -1;
    }

    my $comtype = $args{type};
    my $font = $args{font};
    my $id = (exists($args{id})) ? $args{id} : '';

    if ($comtype !~ /^(real|alias|subst|font)$/) {
	emes($obj, "Unknown type '$comtype'.");
	return -1;
    }

    my @index;
    my ($i, $m);

    if ($id eq '') {
	@index = $obj->grep($comtype, f1 => $font);
    } else {
	@index = $obj->grep($comtype, f0 => $id, f1 => $font);
    }

    return -1 unless(@index);

    foreach $i (@index) {
	$id = $obj->{0}->[$i];

	$obj->{unregistering}->{$i} = '';

	do_update($obj, $id);

	do_unset($obj, $id, $font) if ($Purge);

	$obj->delete($i);
	delete($obj->{unregistering}->{$i});

	my @l = $obj->grep('mark', f0 => $id, f1 => $font);
	if (@l) {
	    $obj->delete(@l);
	}
	    
    }
}

my %Conv = ( id => 'f0', font => 'f1', type => 'f2', priority => 'f3',
	     category => 'f4', depid => 'f5', depfont => 'f6', hints => 'f7');

sub convert_grep_argument {
    my @ret = ();
    
    while (@_ > 0) {
	my $key = shift;
	my $value = shift;

	if (exists($Conv{$key})) {
	    push(@ret, $Conv{$key}, $value);
	} else {
	    push(@ret, $key, $value);
	}
    }

    return @ret;
}

sub defoma_id_get_font {
    my $o = shift;
    my $t = shift;
    my @args = convert_grep_argument(@_);

    return $o->grep($t, @args);
}

sub defoma_id_update {
    my $o = shift;
    my $id = shift;

    do_update($o, $id);
}

1;

