package Debian::Defoma::IdCache;
use strict;
use POSIX;

sub new {
    my $class = shift;

    my $o = {
	0 => [],
	1 => [],
	2 => [],
	3 => [],
	4 => [],
	5 => [],
	6 => [],
	7 => [],
	real => [],
	alias => [],
	mark => [],
	subst => [],
	font => [],
	installed => [],
	_installed => {},
	g0 => {},
	g1 => {},
	file => shift,
	cnt => 0,
	pkg => shift,
	suffix => shift,
	unregistering => {},
	delay => 0,
	callback => 1
    };

    $o->{e_id} = $o->{0};
    $o->{e_font} = $o->{1};
    $o->{e_type} = $o->{2};
    $o->{e_priority} = $o->{3};
    $o->{e_category} = $o->{4};
    $o->{e_depid} = $o->{5};
    $o->{e_depfont} = $o->{6};
    $o->{e_hints} = $o->{7};

    bless $o;
    return $o;
}

sub read {
    my $o = shift;

    my $x;
    my $file = $o->{file};
    my $i = 0;
    my @j;
    if (open(F, $o->{file})) {
	while(<F>) {
	    chomp($_);
	    my @list = split(/ +/, $_);
	    my ($k0, $k1, $k2);

	    if ($list[2] eq 'Ir' || $list[2] eq 'Ia' || $list[2] eq 'IS') {
		@j = $o->grep('', f0 => $list[0], f1 => $list[1]);
		if (@j > 0) {
		    $o->{2}->[$j[0]] .= 'I';
		    $o->{_installed}->{$j[0]} = $x;
		}

		next;
	    }
	    
	    $o->{0}->[$i] = $k0 = shift(@list);
	    $o->{1}->[$i] = $k1 = shift(@list);
	    $o->{2}->[$i] = $k2 = shift(@list);
	    $o->{3}->[$i] = shift(@list);
	    $o->{4}->[$i] = shift(@list);
	    $o->{5}->[$i] = shift(@list);
	    $o->{6}->[$i] = shift(@list);
	    $o->{7}->[$i] = (@list > 0) ? join(' ', @list) : '';

	    $o->{g0}->{$k0} = [] unless (exists($o->{g0}->{$k0}));
	    push(@{$o->{g0}->{$k0}}, $i);
	    
	    $o->{g1}->{$k1} = [] unless (exists($o->{g1}->{$k1}));
	    push(@{$o->{g1}->{$k1}}, $i);
	    
	    if ($k2 =~ /^Sr/) {
		push(@{$o->{real}}, $i);
	    } elsif ($k2 =~ /^Sa/) {
		push(@{$o->{alias}}, $i);
	    } elsif ($k2 =~ /^SS/) {
		push(@{$o->{subst}}, $i);
	    } elsif ($k2 =~ /^M/) {
		push(@{$o->{mark}}, $i);
	    }

	    if ($k2 =~ /..I$/) {
		$o->{_installed}->{$i} = $x;
	    }
	    
	    if ($k2 =~ /^S[Sar]/) {
		push(@{$o->{font}}, $i);
	    }

	    $i++;
	}
	close F;
    }
    $o->{cnt} = $i;

    @{$o->{installed}} = sort {$a <=> $b} (keys(%{$o->{_installed}}));

    return 0;
}

sub write {
    my $o = shift;

    my $file = $o->{file};
    my $max = $o->{cnt};
    my ($i, $j);

    if (open(F, '>' . $file)) {
	for ($i = 0; $i < $max; $i++) {
	    $j = $o->{0}->[$i];
	    if ($j ne '') {
		print F $j, ' ', $o->{1}->[$i], ' ', $o->{2}->[$i], ' ';
		print F $o->{3}->[$i], ' ', $o->{4}->[$i], ' ';
		print F $o->{5}->[$i], ' ', $o->{6}->[$i], ' ';
		print F $o->{7}->[$i], "\n";
	    }
	}
	close F;
    }
    unlink($file) unless(-s $file);

    return 0;
}

sub grep {
    my $o = shift;
    my $t = shift;
    my %op = @_;
    my @pat = ();
    my @idx = ();
    my ($i, $j, $k, $ii, $max, $or, $match, $pmax);
    my @ret = ();
    my @lines = ();
    my $lptr;

    if ($t) {
	@lines = (@{$o->{$t}});

	return @ret if (@lines == 0);
    }

    $or = 0;
    $pmax = 0;
    foreach $i (keys(%op)) {
	if ($i eq 'or') {
	    $or = 1;
	} elsif ($i =~ /(.)(.)/) {
	    $ii = $2;
	    $ii += 8 if ($1 eq 'r');
	    $j = $op{$i};

	    if ($ii <= 1) {
		return @ret unless (exists($o->{'g'.$ii}->{$j}));
		my @l2 = @{$o->{'g'.$ii}->{$j}};
		my @l1 = @lines;

		if (@lines == 0) {
		    @lines = @l2;
		    next;
		}
		
		@lines = ();

		while (@l1 && @l2) {
		    if ($l1[0] < $l2[0]) {
			shift(@l1);
		    } elsif ($l1[0] > $l2[0]) {
			shift(@l2);
		    } else {
			push(@lines, $l1[0]);
			shift(@l1);
			shift(@l2);
		    }
		}
	    } else {
		$idx[$pmax] = $ii;
		$pat[$pmax] = $j;
		
		$pmax++;
	    }
	}
    }

    foreach $i (@lines) {
	next unless ($o->{0}->[$i]);
	
	$match = 1;
	for ($j = 0; $j < $pmax; $j++) {
	    $match = 0;
	    $ii = $idx[$j];
	    if ($ii >= 8) {
		$ii -= 8;
		$match = 1 if ($o->{$ii}->[$i] =~ /$pat[$j]/);
	    } else {
		$match = 1 if ($o->{$ii}->[$i] eq $pat[$j]);
	    }
	    
	    if ($or) {
		last if ($match);
	    } else {
		last if ($match == 0);
	    }
	}
	
	push(@ret, $i) if ($match);
    }
    
    return @ret;
}

sub add {
    my $o = shift;
    my $i = $o->{cnt};
    my $j = 0;


    $o->{g0}->{$_[0]} = [] unless (exists($o->{g0}->{$_[0]}));
    push(@{$o->{g0}->{$_[0]}}, $i);
    
    $o->{g1}->{$_[1]} = [] unless (exists($o->{g1}->{$_[1]}));
    push(@{$o->{g1}->{$_[1]}}, $i);
    
    if ($_[2] eq 'Sr') {
	push(@{$o->{real}}, $i);
    } elsif ($_[2] eq 'Sa') {
	push(@{$o->{alias}}, $i);
    } elsif ($_[2] eq 'SS') {
	push(@{$o->{subst}}, $i);
    } elsif ($_[2] =~ /^M/) {
	push(@{$o->{mark}}, $i);
    }

    if ($_[2] =~ /S[Sar]/) {
	push(@{$o->{font}}, $i);
    }

    my $font = shift;
    for ($j = 1; $j < 8; $j++) {
	$o->{$j}->[$i] = shift;
    }
    $o->{0}->[$i] = $font;

    $o->{cnt}++;

    return $i;
}

sub delete {
    my $o = shift;
    my ($i, $j, $k);
    my @ks = ();

    foreach $i (@_) {
	$o->{0}->[$i] = '';
    }

    return 0;
}

sub install {
    my $o = shift;
    my $i = shift;
    my $x;

    $o->{2}->[$i] .= 'I';

    $o->{_installed}->{$i} = $x;
    @{$o->{installed}} = sort {$a <=> $b} (keys(%{$o->{_installed}}));
}

sub uninstall {
    my $o = shift;
    my $i = shift;

    $o->{2}->[$i] =~ s/I$//;

    delete($o->{_installed}->{$i});
    @{$o->{installed}} = sort {$a <=> $b} (keys(%{$o->{_installed}}));
}

1;
